"use strict";

exports.__esModule = true;
exports.dataObserver = exports.initStore = exports.onDelete = exports.onUpdate = exports.onLoadData = exports.onDisplaceShip = exports.onRemoveShip = exports.onAddShip = exports.onDPInit = exports.reducer = exports.DATA_PATH = exports.PLUGIN_KEY = void 0;

var _fsExtra = require("fs-extra");

var _lodash = require("lodash");

var _path = _interopRequireDefault(require("path"));

var _redux = require("redux");

var _reduxObservers = require("redux-observers");

var _selectors = require("views/utils/selectors");

var _fileWriter = _interopRequireDefault(require("./file-writer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

const PLUGIN_KEY = 'poi-plugin-ship-info';
exports.PLUGIN_KEY = PLUGIN_KEY;
const {
  APPDATA_PATH
} = window;

const DATA_PATH = _path["default"].join(APPDATA_PATH, `${PLUGIN_KEY}.json`);

exports.DATA_PATH = DATA_PATH;
let bookmarkInitState = {};
const plannerInitState = {
  archive: {},
  current: []
};
const uiInitState = {
  activeDropdown: '',
  isExtend: true,
  toTop: true
};

try {
  const initState = JSON.parse(localStorage.getItem(PLUGIN_KEY)) || {};
  bookmarkInitState = initState;
} catch (e) {
  console.error(e.stack);
}

const bookmarkReducer = (state = bookmarkInitState, action) => {
  const {
    type,
    bookmark,
    settings,
    data
  } = action;

  switch (type) {
    case '@@poi-plugin-ship-info@update':
      return { ...state,
        [bookmark]: { ...(0, _lodash.omit)(settings, 'shipTypeChecked'),
          name: bookmark
        }
      };

    case '@@poi-plugin-ship-info@delete':
      {
        return (0, _lodash.omit)(state, bookmark);
      }

    case '@@poi-plugin-ship-info@init':
      {
        return { ...state,
          ...data.bookmark
        };
      }

    default:
      return state;
  }
}; // current: [[ship ids in the area ] for area in fcd ] current deck planner's profile,
// it depends on the fcd name, so only ship ids are stored
// TODO dpBookmarks: historical data, we must make sure it is independent of fcd
// {
//   name: 'example',
//   areas: [
//     {
//       name: 'E1',
//       color: '#0099ff',
//       ships: [1, 2, 3]
//     }
//   ]
// }


const plannerReducer = (state = plannerInitState, action) => {
  const {
    type,
    mapname,
    shipId,
    areaIndex,
    fromAreaIndex,
    toAreaIndex,
    data
  } = action;
  const {
    current
  } = state;

  switch (type) {
    case `@@${PLUGIN_KEY}@init`:
      {
        return { ...state,
          ...data.planner
        };
      }

    case `@@${PLUGIN_KEY}@dp-init`:
      {
        if (current.length < mapname.length) {
          const len = mapname.length - current.length;
          return { ...state,
            current: [...current, ...new Array(len).fill([])]
          };
        }

        if (current.length > mapname.length) {
          const newCurrent = current.slice(0, mapname.length);
          return { ...state,
            current: newCurrent
          };
        }

        break;
      }

    case `@@${PLUGIN_KEY}@dp-addShip`:
      {
        const newCurrent = current.slice();

        if (!newCurrent[areaIndex].includes(shipId)) {
          newCurrent[areaIndex] = [...newCurrent[areaIndex], shipId];
          return { ...state,
            current: newCurrent
          };
        }

        break;
      }

    case `@@${PLUGIN_KEY}@dp-removeship`:
      {
        const newCurrent = current.slice();

        if (newCurrent[areaIndex].includes(shipId)) {
          newCurrent[areaIndex] = newCurrent[areaIndex].filter(id => id !== shipId);
          return { ...state,
            current: newCurrent
          };
        }

        break;
      }

    case `@@${PLUGIN_KEY}@dp-displaceShip`:
      {
        const newCurrent = current.slice();

        if (newCurrent[fromAreaIndex].includes(shipId)) {
          newCurrent[fromAreaIndex] = newCurrent[fromAreaIndex].filter(id => id !== shipId);
          newCurrent[toAreaIndex] = [...newCurrent[toAreaIndex], shipId];
          return { ...state,
            current: newCurrent
          };
        }

        break;
      }

    default:
      return state;
  }

  return state;
};

const uiReducer = (state = uiInitState, action) => {
  const {
    isExtend,
    type,
    toTop,
    activeDropdown
  } = action;

  if (type === `@@${PLUGIN_KEY}@scroll`) {
    return { ...state,
      toTop
    };
  }

  if (type === `@@${PLUGIN_KEY}@active-dropdown`) {
    return { ...state,
      activeDropdown: activeDropdown === state.activeDropdown ? '' : activeDropdown
    };
  }

  if (type === `@@${PLUGIN_KEY}@extend`) {
    return { ...state,
      isExtend
    };
  }

  return state;
};

const readyReducer = (state = false, action) => {
  const {
    type
  } = action;

  if (type === `@@${PLUGIN_KEY}@ready`) {
    return true;
  }

  return state;
};

const reducer = (0, _redux.combineReducers)({
  bookmark: bookmarkReducer,
  planner: plannerReducer,
  ready: readyReducer,
  ui: uiReducer
});
exports.reducer = reducer;

const onDPInit = ({
  color,
  mapname
}) => ({
  color,
  mapname,
  type: `@@${PLUGIN_KEY}@dp-init`
});

exports.onDPInit = onDPInit;

const onAddShip = ({
  shipId,
  areaIndex
}) => ({
  areaIndex,
  shipId,
  type: `@@${PLUGIN_KEY}@dp-addShip`
});

exports.onAddShip = onAddShip;

const onRemoveShip = ({
  shipId,
  areaIndex
}) => ({
  areaIndex,
  shipId,
  type: `@@${PLUGIN_KEY}@dp-removeship`
});

exports.onRemoveShip = onRemoveShip;

const onDisplaceShip = ({
  shipId,
  fromAreaIndex,
  toAreaIndex
}) => ({
  fromAreaIndex,
  shipId,
  toAreaIndex,
  type: `@@${PLUGIN_KEY}@dp-displaceShip`
});

exports.onDisplaceShip = onDisplaceShip;

const onLoadData = ({
  data
}) => ({
  data,
  type: `@@${PLUGIN_KEY}@loadData`
}); // actions


exports.onLoadData = onLoadData;

const onUpdate = ({
  bookmark,
  settings
}) => ({
  bookmark,
  settings,
  type: '@@poi-plugin-ship-info@update'
});

exports.onUpdate = onUpdate;

const onDelete = ({
  bookmark
}) => ({
  bookmark,
  type: '@@poi-plugin-ship-info@delete'
});

exports.onDelete = onDelete;

const initStore = async (dispatch, getState) => {
  try {
    const data = await (0, _fsExtra.readJson)(DATA_PATH);
    const mapname = (0, _lodash.get)(getState(), 'fcd.shbiptag.mapname', []);
    const {
      planner = []
    } = data;

    if (planner.length < mapname.length) {
      const len = mapname.length - planner.length;
      data.planner = [...planner, ...new Array(len).fill([])];
    } else if (planner.length > mapname.length) {
      const newCurrent = planner.slice(0, mapname.length);
      data.planner = newCurrent;
    }

    dispatch({
      data,
      type: '@@poi-plugin-ship-info@init'
    });
  } catch (e) {
    console.error(e.stack);
  } finally {
    dispatch({
      type: '@@poi-plugin-ship-info@ready'
    });
  }
};

exports.initStore = initStore;
const fileWriter = new _fileWriter["default"](); // observers

const dataObserver = (0, _reduxObservers.observer)((0, _selectors.extensionSelectorFactory)(PLUGIN_KEY), (dispatch, current = {}) => {
  // avoid initial state overwrites file
  if (current.ready) {
    fileWriter.write(DATA_PATH, current);
  }
});
exports.dataObserver = dataObserver;