"use strict";

exports.__esModule = true;
exports.uniqueShipCountSelector = exports.kai2ShipSelector = exports.adjustedRemodelChainsSelector = exports.shipUniqueMapSelector = exports.uniqueShipIdsSelector = exports.deckPlannerShipMapSelector = exports.deckPlannerAllShipIdsSelector = exports.deckPlannerAreaSelectorFactory = exports.deckPlannerCurrentSelector = exports.shipMenuDataSelector = exports.shipItemSelectorFactory = exports.sallyAreaSelectorFactory = exports.filterShipIdsSelector = exports.shipInfoFiltersSelector = exports.allShipRowsMapSelector = exports.allShipRowsSelector = exports.shipTableDataSelectorFactory = exports.rawValueConfigSelector = exports.shipFleetIdSelectorFactory = exports.shipFleetIdMapSelector = exports.shipInfoConfigSelector = exports.graphSelector = void 0;

var _lodash = _interopRequireWildcard(require("lodash"));

var _fp = _interopRequireDefault(require("lodash/fp"));

var _reselect = require("reselect");

var _i18next = _interopRequireDefault(require("views/env-parts/i18next"));

var _wanakana = require("wanakana");

var _selectors = require("views/utils/selectors");

var _redux = require("./redux");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

const __ = _i18next["default"].getFixedT(null, ['poi-plugin-ship-info', 'resources']);

const graphSelector = (0, _reselect.createSelector)([_selectors.constSelector], constState => (0, _lodash.keyBy)(constState.$shipgraph, 'api_id'));
exports.graphSelector = graphSelector;
const shipInfoConfigSelector = (0, _reselect.createSelector)([_selectors.configSelector, _selectors.fcdSelector], (config, {
  shiptag = {}
}) => ({
  daihatsuRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.daihatsuRadio', 0),
  exSlotRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.exSlotRadio', 0),
  expeditionRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.expeditionRadio', 0),
  inFleetRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.inFleetRadio', 0),
  lockedRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.lockedRadio', 1),
  lvRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.lvRadio', 2),
  marriedRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.marriedRadio', 0),
  modernizationRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.modernizationRadio', 0),
  pagedLayout: (0, _lodash.get)(config, 'plugin.ShipInfo.pagedLayout', 0),
  rawValue: (0, _lodash.get)(config, 'plugin.ShipInfo.rawValue', false),
  remodelRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.remodelRadio', 0),
  sallyAreaChecked: (0, _lodash.get)(config, 'plugin.ShipInfo.sallyAreaChecked', Array((shiptag.mapname || []).length + 1).fill(true)),
  sortName: (0, _lodash.get)(config, 'plugin.ShipInfo.sortName', 'lv'),
  sortOrder: (0, _lodash.get)(config, 'plugin.ShipInfo.sortOrder', 0),
  sparkleRadio: (0, _lodash.get)(config, 'plugin.ShipInfo.sparkleRadio', 0)
}));
exports.shipInfoConfigSelector = shipInfoConfigSelector;
const allFleetShipIdSelector = (0, _reselect.createSelector)([...[...Array(4).keys()].map(fleetId => (0, _selectors.fleetShipsIdSelectorFactory)(fleetId))], (id1, id2, id3, id4) => [id1, id2, id3, id4]);
const shipFleetIdMapSelector = (0, _reselect.createSelector)([_selectors.shipsSelector, allFleetShipIdSelector], (ships, fleetIds) => (0, _lodash.mapValues)(ships, ship => (0, _lodash.findIndex)(fleetIds, fleetId => (0, _lodash.includes)(fleetId, ship.api_id))));
exports.shipFleetIdMapSelector = shipFleetIdMapSelector;
const shipFleetIdSelectorFactory = (0, _lodash.memoize)(shipId => (0, _reselect.createSelector)([shipFleetIdMapSelector], fleetIdMap => fleetIdMap[shipId]));
exports.shipFleetIdSelectorFactory = shipFleetIdSelectorFactory;
const rawValueConfigSelector = (0, _reselect.createSelector)([_selectors.configSelector], config => (0, _lodash.get)(config, 'plugin.ShipInfo.rawValue', false));
exports.rawValueConfigSelector = rawValueConfigSelector;
const shipTableDataSelectorFactory = (0, _lodash.memoize)(shipId => (0, _reselect.createSelector)([(0, _selectors.shipDataSelectorFactory)(shipId), (0, _selectors.shipEquipDataSelectorFactory)(shipId), _selectors.constSelector, shipFleetIdMapSelector, rawValueConfigSelector, _selectors.inRepairShipsIdSelector, _selectors.wctfSelector], ([ship, $ship] = [], equips, {
  $shipTypes
}, fleetIdMap, rawValue, repairs = [], db) => (0, _utils.getShipInfoData)(ship, $ship, equips, $shipTypes, fleetIdMap, rawValue, repairs, db)));
exports.shipTableDataSelectorFactory = shipTableDataSelectorFactory;

const handleTypeFilter = (typeId, shipTypes) => (shipTypes || []).includes(typeId);

const handleLockedFilter = (locked, lockedRadio) => {
  switch (lockedRadio) {
    case 1:
      return locked === 1;

    case 2:
      return locked === 0;

    case 0:
    default:
      return true;
  }
};

const handleExpeditionFilter = (id, expeditionShips, expeditionRadio) => {
  switch (expeditionRadio) {
    case 1:
      return (expeditionShips || []).includes(id);

    case 2:
      return !(expeditionShips || []).includes(id);

    case 0:
    default:
      return true;
  }
};

const handleModernizationFilter = (isCompleted, modernizationRadio) => {
  switch (modernizationRadio) {
    case 1:
      return isCompleted;

    case 2:
      return !isCompleted;

    case 0:
    default:
      return true;
  }
};

const handleRemodelFilter = (after, remodelRadio) => {
  const remodelable = after !== 0;

  switch (remodelRadio) {
    case 1:
      return remodelable;

    case 2:
      return !remodelable;

    case 0:
    default:
      return true;
  }
};

const handleSallyAreaFilter = (sallyArea, sallyAreaChecked) => {
  const checkedAll = (sallyAreaChecked || []).reduce((all, checked) => all && checked, true);

  if (checkedAll) {
    return true;
  }

  return typeof sallyArea !== 'undefined' ? (sallyAreaChecked || [])[sallyArea || 0] : true;
};

const handleInFleetFilter = (fleetId, inFleetRadio) => {
  const isInFleet = fleetId > -1;

  switch (inFleetRadio) {
    case 1:
      return isInFleet;

    case 2:
      return !isInFleet;

    case 0:
    default:
      return true;
  }
};

const handleSparkleFilter = (cond, sparkleRadio) => {
  switch (sparkleRadio) {
    case 1:
      return cond >= 50;

    case 2:
      return cond < 50;

    case 0:
    default:
      return true;
  }
};

const handleExSlotFilter = (exslot, exSlotRadio) => {
  switch (exSlotRadio) {
    case 1:
      return exslot !== 0;

    case 2:
      return exslot === 0;

    case 0:
    default:
      return true;
  }
};

const handleDaihatsuFilter = (daihatsu, daihatsuRadio) => {
  switch (daihatsuRadio) {
    case 1:
      return daihatsu;

    case 2:
      return !daihatsu;

    case 0:
    default:
      return true;
  }
};

const getSortFunction = sortName => {
  switch (sortName) {
    case 'id':
      return ship => ship.id;

    case 'name':
      return [ship => (0, _utils.katakanaToHiragana)(ship.yomi), ship => ship.lv, ship => -ship.id];

    case 'lv':
      // Sort rule of level in game (descending):
      // 1. level (descending)
      // 2. sortno (ascending)
      // 3. id (descending)
      return [ship => ship.lv, ship => -ship.sortno, ship => -ship.id];

    case 'type':
      return [ship => ship.typeId, ship => -ship.sortno, ship => ship.lv, ship => -ship.id];

    case 'hp':
      return [ship => ship.maxhp, ship => -ship.sortno, ship => -ship.id];

    default:
      return [ship => ship[sortName], ship => ship.sortno, ship => -ship.id];
  }
};

const shipTypesSelecor = (0, _reselect.createSelector)([state => (0, _lodash.get)(state, 'const.$shipTypes', {}), state => (0, _lodash.get)(state.config, 'plugin.ShipInfo.shipTypes')], ($shipTypes, shipTypeChecked) => {
  const checked = (0, _utils.intToBoolArray)(shipTypeChecked);

  if (checked.length !== Object.keys($shipTypes).length) {
    return Object.keys($shipTypes).map(s => +s);
  }

  return checked.reduce((types, check, index) => check && index + 1 in $shipTypes ? types.concat([index + 1]) : types, []);
});
const fleetShipsInExpeditionSelectorFactory = (0, _lodash.memoize)(fleetId => (0, _reselect.createSelector)([(0, _selectors.fleetInExpeditionSelectorFactory)(fleetId), (0, _selectors.fleetShipsIdSelectorFactory)(fleetId)], (inExpedition, id) => inExpedition ? id : []));
const expeditionShipsSelector = (0, _reselect.createSelector)([state => fleetShipsInExpeditionSelectorFactory(0)(state), state => fleetShipsInExpeditionSelectorFactory(1)(state), state => fleetShipsInExpeditionSelectorFactory(2)(state), state => fleetShipsInExpeditionSelectorFactory(3)(state)], (...ids) => [].concat(...ids));
const allShipRowsSelector = (0, _reselect.createSelector)([_selectors.shipsSelector, _selectors.stateSelector], (ships, state) => (0, _lodash.map)(ships, ship => shipTableDataSelectorFactory(ship.api_id)(state)));
exports.allShipRowsSelector = allShipRowsSelector;
const allShipRowsMapSelector = (0, _reselect.createSelector)([_selectors.shipsSelector, _selectors.stateSelector], (ships, state) => (0, _lodash.mapValues)(ships, ship => shipTableDataSelectorFactory(ship.api_id)(state)));
exports.allShipRowsMapSelector = allShipRowsMapSelector;
const shipInfoFiltersSelector = (0, _reselect.createSelector)([_selectors.configSelector, _selectors.fcdSelector], (config, {
  shiptag = {}
}) => ({
  maxLevel: (0, _lodash.get)(config, 'plugin.ShipInfo.filters.maxLevel', 10000),
  minLevel: (0, _lodash.get)(config, 'plugin.ShipInfo.filters.minLevel', 1)
}));
exports.shipInfoFiltersSelector = shipInfoFiltersSelector;
const filterShipIdsSelector = (0, _reselect.createSelector)([allShipRowsSelector, shipTypesSelecor, expeditionShipsSelector, shipInfoConfigSelector, shipInfoFiltersSelector], (ships, shipTypes, expeditionShips, {
  lockedRadio,
  expeditionRadio,
  modernizationRadio,
  remodelRadio,
  inFleetRadio,
  exSlotRadio,
  sparkleRadio,
  daihatsuRadio,
  sallyAreaChecked,
  sortName,
  sortOrder
}, {
  maxLevel,
  minLevel
}) => _fp["default"].flow(_fp["default"].filter(ship => handleTypeFilter(ship.typeId, shipTypes) && ship.lv >= Math.min(minLevel, maxLevel) && ship.lv <= Math.max(minLevel, maxLevel) && handleLockedFilter(ship.locked, lockedRadio) && handleExpeditionFilter(ship.id, expeditionShips, expeditionRadio) && handleModernizationFilter(ship.isCompleted, modernizationRadio) && handleRemodelFilter(ship.after, remodelRadio) && handleSallyAreaFilter(ship.sallyArea, sallyAreaChecked) && handleInFleetFilter(ship.fleetId, inFleetRadio) && handleExSlotFilter(ship.exslot, exSlotRadio) && handleSparkleFilter(ship.cond, sparkleRadio) && handleDaihatsuFilter(ship.daihatsu, daihatsuRadio)), _fp["default"].sortBy(getSortFunction(sortName)), sortOrder ? ship => ship : _fp["default"].reverse, _fp["default"].map('id'))(ships));
exports.filterShipIdsSelector = filterShipIdsSelector;
const sallyAreaSelectorFactory = (0, _lodash.memoize)(area => (0, _reselect.createSelector)([_selectors.fcdSelector], fcd => ({
  color: (0, _lodash.get)(fcd, `shiptag.color.${area - 1}`, ''),
  mapname: (0, _lodash.get)(fcd, `shiptag.mapname.${area - 1}`, __('unknown_area', {
    area
  }))
})));
exports.sallyAreaSelectorFactory = sallyAreaSelectorFactory;
const shipItemSelectorFactory = (0, _lodash.memoize)(shipId => (0, _reselect.createSelector)([(0, _selectors.shipDataSelectorFactory)(shipId), _selectors.fcdSelector], ([ship, $ship] = [], {
  shipavatar
} = {}) => !!ship && !!$ship ? {
  area: ship.api_sally_area,
  avatarOffset: (0, _lodash.get)(shipavatar, ['marginMagics', $ship.api_id, 'normal'], 0.555),
  exp: ship.api_exp[0],
  id: ship.api_id,
  lv: ship.api_lv,
  name: $ship.api_name,
  romaji: (0, _wanakana.toRomaji)($ship.api_yomi),
  shipId: $ship.api_id,
  superTypeIndex: _utils.reverseSuperTypeMap[$ship.api_stype] || 0,
  typeId: $ship.api_stype,
  yomi: $ship.api_yomi
} : undefined));
exports.shipItemSelectorFactory = shipItemSelectorFactory;
const shipMenuDataSelector = (0, _reselect.createSelector)([_selectors.shipsSelector, state => state], (_ships, state) => _fp["default"].flow(_fp["default"].map(ship => ship.api_id), _fp["default"].map(shipId => shipItemSelectorFactory(shipId)(state)))(_ships));
exports.shipMenuDataSelector = shipMenuDataSelector;
const deckPlannerCurrentSelector = (0, _reselect.createSelector)([(0, _selectors.extensionSelectorFactory)(_redux.PLUGIN_KEY)], state => (state.planner || {}).current || []);
exports.deckPlannerCurrentSelector = deckPlannerCurrentSelector;
const deckPlannerAreaSelectorFactory = (0, _lodash.memoize)(areaIndex => (0, _reselect.createSelector)([deckPlannerCurrentSelector], current => current[areaIndex] || []));
exports.deckPlannerAreaSelectorFactory = deckPlannerAreaSelectorFactory;
const deckPlannerAllShipIdsSelector = (0, _reselect.createSelector)([deckPlannerCurrentSelector], current => (0, _lodash.flatten)(current));
exports.deckPlannerAllShipIdsSelector = deckPlannerAllShipIdsSelector;
const deckPlannerShipMapSelector = (0, _reselect.createSelector)([deckPlannerCurrentSelector], current => (0, _lodash.fromPairs)((0, _lodash.flatMap)(current, (ships, areaIndex) => ships.map(id => [id, areaIndex]))));
exports.deckPlannerShipMapSelector = deckPlannerShipMapSelector;
const ourShipsSelector = (0, _reselect.createSelector)([_selectors.constSelector], ({
  $ships = {}
} = {}) => (0, _lodash["default"])($ships).pickBy(({
  api_sortno
}) => Boolean(api_sortno)).value());
const beforeShipMapSelector = (0, _reselect.createSelector)([ourShipsSelector], $ships => (0, _lodash["default"])($ships).filter(ship => +(ship.api_aftershipid || 0) > 0).map(ship => [ship.api_aftershipid, ship.api_id]).fromPairs().value()); // the chain starts from each ship, thus incomplete if the ship is not the starting one
// the adjustedRemodelChainsSelector will return complete chains for all ships

const remodelChainsSelector = (0, _reselect.createSelector)([ourShipsSelector], $ships => (0, _lodash.mapValues)($ships, ({
  api_id: shipId
}) => {
  let current = $ships[shipId];
  let next = +(current.api_aftershipid || 0);
  let same = [shipId];

  while (!same.includes(next) && next > 0) {
    same = [...same, next];
    current = $ships[next] || {};
    next = +(current.api_aftershipid || 0);
  }

  return same;
}));
const uniqueShipIdsSelector = (0, _reselect.createSelector)([ourShipsSelector, beforeShipMapSelector], ($ships, beforeShipMap) => (0, _lodash["default"])($ships).filter(({
  api_id
}) => !(api_id in beforeShipMap)).map(({
  api_id
}) => api_id).value());
exports.uniqueShipIdsSelector = uniqueShipIdsSelector;
const shipUniqueMapSelector = (0, _reselect.createSelector)([uniqueShipIdsSelector, remodelChainsSelector], (shipIds, chains) => (0, _lodash["default"])(shipIds).flatMap(shipId => (0, _lodash["default"])(chains[shipId]).map(id => [id, shipId]).value()).fromPairs().value());
exports.shipUniqueMapSelector = shipUniqueMapSelector;
const adjustedRemodelChainsSelector = (0, _reselect.createSelector)([remodelChainsSelector, shipUniqueMapSelector], (remodelChains, uniqueMap) => (0, _lodash["default"])(uniqueMap).mapValues(uniqueId => remodelChains[uniqueId]).value());
exports.adjustedRemodelChainsSelector = adjustedRemodelChainsSelector;
const kai2ShipSelector = (0, _reselect.createSelector)([uniqueShipIdsSelector, adjustedRemodelChainsSelector], (ships, remodelChains) => _fp["default"].flow(_fp["default"].filter(shipId => remodelChains[shipId].length > 3))(ships));
exports.kai2ShipSelector = kai2ShipSelector;
const uniqueShipCountSelector = (0, _reselect.createSelector)([uniqueShipIdsSelector, adjustedRemodelChainsSelector, shipMenuDataSelector], (uniqs, remodelChains, ships) => (0, _lodash.mapValues)(remodelChains, shipIds => _fp["default"].filter(ship => shipIds.includes(ship.shipId))(ships).length));
exports.uniqueShipCountSelector = uniqueShipCountSelector;