"use strict";

exports.__esModule = true;
exports.Export = void 0;

var _core = require("@blueprintjs/core");

var _electron = require("electron");

var _fsExtra = require("fs-extra");

var _lodash = require("lodash");

var _react = _interopRequireWildcard(require("react"));

var _reactFontawesome = _interopRequireDefault(require("react-fontawesome"));

var _reactI18next = require("react-i18next");

var _overlay = require("views/components/etc/overlay");

var _selectors = require("../../selectors");

var _checkbox = require("../components/checkbox");

var _layout = require("../components/layout");

var _buildCsv = require("./build-csv");

var _temp;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

const {
  dialog
} = _electron.remote;
const isWin = process.platform === 'win32';

const Select = ({
  label,
  options,
  value,
  onChange
}) => {
  const handleChange = (0, _react.useCallback)(e => {
    onChange(e.currentTarget.value);
  }, [onChange]);
  const {
    t
  } = (0, _reactI18next.useTranslation)('poi-plugin-ship-info');
  return _react["default"].createElement(_checkbox.Checkbox, null, _react["default"].createElement(_checkbox.CheckboxLabel, null, t(label)), _react["default"].createElement(_core.HTMLSelect, {
    value: value,
    onChange: handleChange
  }, (0, _lodash.map)(options, ({
    name,
    value: V
  }) => _react["default"].createElement("option", {
    key: name,
    value: V
  }, t(name)))));
};

const sepOptions = [{
  name: 'comma',
  value: ','
}, {
  name: 'semicolon',
  value: ';'
}, {
  name: 'tab',
  value: '\t'
}];
const endOptions = [{
  name: 'Windows (CRLF)',
  value: '\r\n'
}, {
  name: 'Unix (LF)',
  value: '\n'
}];
const selectionOptions = [{
  name: 'Current list',
  value: 'current'
}, {
  name: 'All',
  value: 'all'
}];
const ExportContent = (0, _reactI18next.withTranslation)(['poi-plugin-ship-info'])((_temp = class ExportContentBase extends _react.Component {
  constructor(...args) {
    super(...args);
    this.state = {
      end: isWin ? '\r\n' : '\n',
      selection: 'current',
      sep: ','
    };

    this.handleChange = name => value => () => {
      this.setState({
        [name]: value
      });
    };

    this.getShipData = () => {
      const {
        selection
      } = this.state;
      const state = window.getStore();
      const ids = (0, _selectors.filterShipIdsSelector)(state);
      const ships = (0, _selectors.allShipRowsMapSelector)(state);
      return selection === 'current' ? (0, _lodash.map)(ids, id => ships[id]) : (0, _lodash.values)(ships);
    };

    this.handleExportToClipboard = () => {
      const {
        sep,
        end
      } = this.state;

      _electron.clipboard.writeText((0, _buildCsv.buildCsv)(this.getShipData(), sep, end));
    };

    this.handleExportToFile = () => {
      const {
        t
      } = this.props;

      const bw = _electron.remote.getCurrentWindow();

      dialog.showSaveDialog(bw, {
        filters: [{
          extensions: ['csv'],
          name: 'CSV file'
        }],
        title: t('Position the file to save into')
      }, async filename => {
        if (filename) {
          const {
            sep,
            end
          } = this.state;

          try {
            await (0, _fsExtra.outputFile)(filename, (0, _buildCsv.buildCsv)(this.getShipData(), sep, end));
          } catch (e) {
            console.error(e);
          }
        }
      });
    };
  }

  render() {
    const {
      sep,
      end,
      selection
    } = this.state;
    const {
      t
    } = this.props;
    return _react["default"].createElement(_layout.Container, null, _react["default"].createElement("div", null, _react["default"].createElement(Select, {
      label: "Data",
      options: selectionOptions,
      value: selection,
      onChange: this.handleChange('selection')
    }), _react["default"].createElement(Select, {
      label: "Separator",
      options: sepOptions,
      value: sep,
      onChange: this.handleChange('sep')
    }), _react["default"].createElement(Select, {
      label: "Line break",
      options: endOptions,
      value: end,
      onChange: this.handleChange('end')
    })), _react["default"].createElement("div", null, _react["default"].createElement(_core.ButtonGroup, null, _react["default"].createElement(_core.Button, {
      onClick: this.handleExportToClipboard
    }, t('Copy to clipboard')), _react["default"].createElement(_core.Button, {
      onClick: this.handleExportToFile
    }, t('Export to file')))));
  }

}, _temp));

const Export = () => {
  return _react["default"].createElement(_overlay.Popover, {
    hasBackdrop: true
  }, _react["default"].createElement(_core.Button, {
    minimal: true
  }, _react["default"].createElement(_reactFontawesome["default"], {
    name: "download"
  })), _react["default"].createElement(ExportContent, null));
};

exports.Export = Export;