"use strict";

exports.__esModule = true;
exports.Planner = void 0;

var _core = require("@blueprintjs/core");

var _classnames = _interopRequireDefault(require("classnames"));

var _lodash = require("lodash");

var _polished = require("polished");

var _react = _interopRequireWildcard(require("react"));

var _reactFontawesome = _interopRequireDefault(require("react-fontawesome"));

var _reactI18next = require("react-i18next");

var _reactRedux = require("react-redux");

var _reduxObservers = require("redux-observers");

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _overlay = require("views/components/etc/overlay");

var _windowEnv = require("views/components/etc/window-env");

var _createStore = require("views/create-store");

var _redux = require("../../redux");

var _selectors = require("../../selectors");

var _utils = require("../../utils");

var _checkbox = require("../components/checkbox");

var _area = require("./area");

var _shipGrid = require("./ship-grid");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// tslint:disable jsx-no-lambda
const reorderShips = (dispatch, getState) => {
  const state = getState();
  const ships = (0, _selectors.shipMenuDataSelector)(state);
  const planMap = (0, _selectors.deckPlannerShipMapSelector)(state);
  (0, _lodash.each)(ships, ship => {
    if (ship.area > 0) {
      if (!(ship.id in planMap)) {
        dispatch((0, _redux.onAddShip)({
          areaIndex: ship.area - 1,
          shipId: ship.id
        }));
      } else if (ship.area - 1 !== planMap[ship.id]) {
        dispatch((0, _redux.onDisplaceShip)({
          fromAreaIndex: planMap[ship.id],
          shipId: ship.id,
          toAreaIndex: ship.area - 1
        }));
      }
    }
  });
  (0, _lodash.each)(Object.keys(planMap), shipId => {
    if ((0, _lodash.findIndex)(ships, ship => ship.id === +shipId) < 0) {
      dispatch((0, _redux.onRemoveShip)({
        areaIndex: planMap[shipId],
        shipId: +shipId
      }));
    }
  });
};

const Palette = _styledComponents["default"].div.withConfig({
  displayName: "planner__Palette",
  componentId: "sc-1itvfpl-0"
})(["position:sticky;top:0;z-index:20;background-color:", ";padding-left:1em;border-left:8px solid ", ";:hover{background-color:", ";}"], props => (0, _polished.rgba)(props.theme.variant === 'dark' ? '#000' : '#fff', 0.75), props => props.color || '#000', props => props.theme.variant === 'dark' ? '#000' : '#fff');

const ActionPanel = _styledComponents["default"].div.withConfig({
  displayName: "planner__ActionPanel",
  componentId: "sc-1itvfpl-1"
})(["display:flex;justify-content:center;align-items:center;"]);

const PlannerContent = (0, _reactRedux.connect)(state => {
  const displayFleetName = (0, _lodash.get)(state.config, 'plugin.ShipInfo.displayFleetName', false);
  const mapname = displayFleetName ? (0, _lodash.get)(state, ['fcd', 'shiptag', 'fleetname', window.language], []) : (0, _lodash.get)(state, ['fcd', 'shiptag', 'mapname'], []);
  return {
    color: (0, _lodash.get)(state, 'fcd.shiptag.color', []),
    current: (0, _selectors.deckPlannerCurrentSelector)(state),
    displayFleetName,
    mapname,
    vibrant: (0, _lodash.get)(state, 'config.poi.vibrant')
  };
})(({
  mapname,
  color,
  current,
  displayFleetName,
  dispatch,
  activeTab
}) => {
  (0, _react.useEffect)(() => {
    if (current.length !== mapname.length) {
      dispatch((0, _redux.onDPInit)({
        color,
        mapname
      }));
    }

    dispatch(reorderShips);
  }); // const handleCaptureImage = useCallback(() => {
  //   captureRect()
  // })

  const {
    t
  } = (0, _reactI18next.useTranslation)(['poi-plugin-ship-info']); // public handleCaptureImage = async () => {
  //   this.setState({ extend: true }, async () => {
  //     await captureRect('#planner-rect', this.props.window.document)
  //     this.setState({ extend: false })
  //   })
  // }

  const handleChangeDisplayFleetName = (0, _react.useCallback)(() => {
    window.config.set('plugin.ShipInfo.displayFleetName', !displayFleetName);
  }, [displayFleetName]);
  const handleRefresh = (0, _react.useCallback)(() => {
    dispatch(reorderShips);
  }, [dispatch]);
  const areas = mapname.map((name, index) => ({
    areaIndex: index,
    color: color[index],
    name,
    ships: []
  }));
  const [fill, setFill] = (0, _react.useState)(-1);
  const captureSection = (0, _react.useRef)(null);
  const {
    window
  } = (0, _react.useContext)(_windowEnv.WindowEnv);
  const capture = (0, _react.useCallback)(() => {
    if (captureSection.current) {
      (0, _utils.captureRect)(captureSection.current);
    }
  }, []);
  return _react["default"].createElement("div", null, _react["default"].createElement(ActionPanel, null, _react["default"].createElement(_core.Button, {
    intent: _core.Intent.PRIMARY,
    minimal: true,
    onClick: handleRefresh
  }, t('Refresh')), !window.isMain && _react["default"].createElement(_core.Button, {
    intent: _core.Intent.PRIMARY,
    minimal: true,
    onClick: capture
  }, t('Save to image'))), _react["default"].createElement(_core.Callout, {
    intent: _core.Intent.PRIMARY
  }, t('helpContent')), activeTab === 'area' ? _react["default"].createElement("div", null, _react["default"].createElement("div", {
    ref: captureSection
  }, areas.map(area => _react["default"].createElement(_area.Area, {
    key: area.color,
    area: area,
    index: area.areaIndex,
    others: areas.filter(({
      areaIndex
    }) => areaIndex !== area.areaIndex)
  })))) : _react["default"].createElement("div", null, _react["default"].createElement(Palette, {
    color: color[fill]
  }, _react["default"].createElement(_checkbox.Checkbox, null, _react["default"].createElement(_checkbox.CheckboxLabel, null, t('Palette')), _react["default"].createElement(_checkbox.CheckboxOption, {
    key: -1,
    checked: fill === -1,
    onClick: () => setFill(-1)
  }, t('None')), areas.map(area => _react["default"].createElement(_checkbox.CheckboxOption, {
    key: area.color,
    onClick: () => setFill(area.areaIndex),
    checked: fill === area.areaIndex
  }, area.name)))), _react["default"].createElement(_shipGrid.ShipGrid, {
    fill: fill,
    ref: captureSection
  })));
});
const PlannerDialog = (0, _styledComponents["default"])(_overlay.Dialog).withConfig({
  displayName: "planner__PlannerDialog",
  componentId: "sc-1itvfpl-2"
})(["width:80vw;height:90vh;", "{overflow-y:scroll;}"], `.${_core.Classes.DIALOG_BODY}`);

const Title = _styledComponents["default"].div.withConfig({
  displayName: "planner__Title",
  componentId: "sc-1itvfpl-3"
})(["display:flex;padding-right:2em;"]);

const Heading = _styledComponents["default"].div.withConfig({
  displayName: "planner__Heading",
  componentId: "sc-1itvfpl-4"
})(["flex-grow:1;"]);

const Planner = () => {
  const [isOpen, setIsOpen] = (0, _react.useState)(false);
  const {
    t
  } = (0, _reactI18next.useTranslation)('poi-plugin-ship-info');
  const [activeTab, setActiveTab] = (0, _react.useState)('area');
  (0, _react.useEffect)(() => {
    const unsubscribe = (0, _reduxObservers.observe)(_createStore.store, [_redux.dataObserver]);
    return () => unsubscribe();
  }, []);
  return _react["default"].createElement(_react["default"].Fragment, null, _react["default"].createElement(_core.Button, {
    minimal: true,
    onClick: () => setIsOpen(true)
  }, _react["default"].createElement(_reactFontawesome["default"], {
    name: "tags"
  })), _react["default"].createElement(PlannerDialog, {
    isOpen: isOpen,
    autoFocus: true,
    canOutsideClickClose: true,
    onClose: () => setIsOpen(false),
    title: _react["default"].createElement(Title, null, _react["default"].createElement(Heading, null, t('Deck Planner')), _react["default"].createElement(_core.Tabs, {
      id: "deck-planner-content",
      selectedTabId: activeTab,
      onChange: tab => setActiveTab(tab)
    }, _react["default"].createElement(_core.Tab, {
      id: "area",
      title: t('Area View')
    }), _react["default"].createElement(_core.Tab, {
      id: "grid",
      title: t('Ship Grid')
    })))
  }, _react["default"].createElement("div", {
    className: (0, _classnames["default"])(_core.Classes.DIALOG_BODY, 'ship-info-scrollable')
  }, _react["default"].createElement(PlannerContent, {
    activeTab: activeTab
  }))));
};

exports.Planner = Planner;