"use strict";

exports.__esModule = true;
exports.ShipItem = void 0;

var _lodash = require("lodash");

var _polished = require("polished");

var _react = _interopRequireDefault(require("react"));

var _reactFontawesome = _interopRequireDefault(require("react-fontawesome"));

var _reactI18next = require("react-i18next");

var _reactRedux = require("react-redux");

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _shipImg = require("views/utils/ship-img");

var _selectors = require("../../selectors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

const ShipAvatar = _styledComponents["default"].img.attrs(({
  left
}) => ({
  style: {
    left: `-${left}px`
  }
})).withConfig({
  displayName: "ship-item__ShipAvatar",
  componentId: "sc-1j6vyfd-0"
})(["height:40px;position:absolute;top:0px;"]);

const Gradient = _styledComponents["default"].div.withConfig({
  displayName: "ship-item__Gradient",
  componentId: "sc-1j6vyfd-1"
})(["height:40px;width:100%;position:absolute;top:0px;left:0px;"]);

const ShipGridCell = _styledComponents["default"].div.attrs({
  role: 'button',
  tabIndex: 0
}).withConfig({
  displayName: "ship-item__ShipGridCell",
  componentId: "sc-1j6vyfd-2"
})(["width:calc(100% / 7 - 4px);min-width:6em;max-width:16em;margin:2px;background-color:rgba(255,255,255,0.1);text-align:right;background-size:auto 40px;background-repeat:no-repeat;height:40px;position:relative;overflow:hidden;@media (min-width:1440px){width:calc(100% / 10 - 4px);}", "{z-index:0;}", "{z-index:0;background:", ";}&:hover{", "{z-index:5;}}}"], ShipAvatar, Gradient, props => `linear-gradient(to right, transparent, ${(0, _polished.rgba)(props.color, props.useAvatar ? 0.5 : 0.75)})`, ShipAvatar);

const Text = _styledComponents["default"].div.withConfig({
  displayName: "ship-item__Text",
  componentId: "sc-1j6vyfd-3"
})(["z-index:10;position:absolute;right:0;width:100%;overflow:hidden;"]);

const ShipName = _styledComponents["default"].div.withConfig({
  displayName: "ship-item__ShipName",
  componentId: "sc-1j6vyfd-4"
})(["font-weight:600;display:inline-block;padding:0 1ex;color:white;user-select:none;max-width:75%;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;"]);

const ShipLevel = _styledComponents["default"].div.withConfig({
  displayName: "ship-item__ShipLevel",
  componentId: "sc-1j6vyfd-5"
})(["display:inline-block;line-height:1em;padding:0 1ex;color:white;user-select:none;"]);

const ShipItem = (0, _reactRedux.connect)(state => ({
  color: (0, _lodash.get)(state, 'fcd.shiptag.color', []),
  ip: (0, _lodash.get)(state, 'info.server.ip', '203.104.209.71'),
  mapname: (0, _lodash.get)(state, 'fcd.shiptag.mapname', []),
  planMap: (0, _selectors.deckPlannerShipMapSelector)(state),
  useAvatar: (0, _lodash.get)(state, ['config', 'poi', 'appearance', 'avatar'], true)
}))(({
  ship,
  planMap,
  color,
  onClick,
  onContextmenu,
  ip,
  useAvatar
}) => {
  const {
    t
  } = (0, _reactI18next.useTranslation)('resources');
  const bgColor = color[ship.area - 1] || color[planMap[ship.id]] || '#000';
  return _react["default"].createElement(ShipGridCell, {
    color: bgColor,
    useAvatar: useAvatar,
    onClick: !(ship.area > 0) ? onClick : undefined,
    onContextMenu: !(ship.area > 0) ? onContextmenu : undefined
  }, useAvatar && _react["default"].createElement(ShipAvatar, {
    src: (0, _shipImg.getShipImgPath)(ship.shipId, 'remodel', false, ip),
    left: Math.round(ship.avatarOffset * 40)
  }), _react["default"].createElement(Gradient, null), _react["default"].createElement(Text, null, _react["default"].createElement(ShipLevel, null, "Lv.", ship.lv), _react["default"].createElement("br", null), _react["default"].createElement(ShipName, null, t(ship.name), ship.area > 0 && _react["default"].createElement(_react["default"].Fragment, null, ' ', _react["default"].createElement(_reactFontawesome["default"], {
    name: "tag"
  })))));
});
exports.ShipItem = ShipItem;