"use strict";

exports.__esModule = true;
exports.Cells = exports.Cell = void 0;

var _lodash = require("lodash");

var _path = _interopRequireDefault(require("path"));

var _react = _interopRequireDefault(require("react"));

var _reactFontawesome = _interopRequireDefault(require("react-fontawesome"));

var _reactI18next = require("react-i18next");

var _reactRedux = require("react-redux");

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _avatar = require("views/components/etc/avatar");

var _overlay = require("views/components/etc/overlay");

var _tools = require("views/utils/tools");

var _polished = require("polished");

var _constants = require("../constants");

var _utils = require("../utils");

var _sallyArea = require("./sally-area");

var _slotitems = require("./slotitems");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const TRANSPARENCY = 0.5;

const Cell = _styledComponents["default"].div.withConfig({
  displayName: "cells__Cell",
  componentId: "hq8fj8-0"
})(["white-space:nowrap;line-height:35px;vertical-align:middle;overflow:visible;text-overflow:ellipsis;cursor:default;display:flex;align-items:center;background-color:", ";"], props => props.isEven && (0, _polished.rgba)(props.theme.BLUE5, 0.05));

exports.Cell = Cell;

const enableAvatarConfigSelector = state => (0, _lodash.get)(state, ['config', 'poi', 'appearance', 'avatar'], true);

const Id = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.id);

const Name = ({
  ship,
  enableAvatar,
  ...props
}) => {
  const {
    t
  } = (0, _reactI18next.useTranslation)(['resources']);
  return _react["default"].createElement(Cell, props, enableAvatar && _react["default"].createElement(_avatar.Avatar, {
    mstId: ship.shipId,
    height: 35
  }), _react["default"].createElement("span", {
    title: t(ship.name)
  }, t(ship.name)), ship.fleetId > -1 && _react["default"].createElement("img", {
    alt: `fleet: ${ship.fleetId + 1}`,
    height: 16,
    src: (0, _utils.fileUrl)(_path["default"].resolve(__dirname, `../../assets/images/fleet/${ship.fleetId + 1}.png`))
  }), _react["default"].createElement(_sallyArea.SallyArea, {
    area: ship.sallyArea,
    info_id: ship.id
  }));
};

const Type = ({
  ship,
  ...props
}) => {
  const {
    t
  } = (0, _reactI18next.useTranslation)(['resources']);
  return _react["default"].createElement(Cell, props, window.language === 'en-US' ? _utils.shipTypes[ship.typeId] : t(ship.type));
};

const Soku = ({
  ship,
  ...props
}) => {
  const {
    soku
  } = ship;
  const sokuString = _constants.sokuInterpretation[soku] || 'Unknown';
  const {
    t
  } = (0, _reactI18next.useTranslation)(['poi-plugin-ship-info']);
  return _react["default"].createElement(Cell, props, _react["default"].createElement("span", null, t(sokuString)));
};

const Lv = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.lv);

const Cond = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.cond);

const Hp = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, _react["default"].createElement("span", null, ship.taik[0], ship.maxhp - ship.taik[0] > 0 && _react["default"].createElement("sup", null, "+", ship.maxhp - ship.taik[0])));

const KaryokuCell = (0, _styledComponents["default"])(Cell).withConfig({
  displayName: "cells__KaryokuCell",
  componentId: "hq8fj8-1"
})(["background-color:", ";"], props => props.max && (0, _polished.rgba)(props.theme.RED3, TRANSPARENCY));

const Karyoku = ({
  ship,
  ...props
}) => {
  const {
    karyoku,
    karyokuMax,
    karyokuNow
  } = ship;
  const karyokuString = karyokuNow >= karyokuMax ? 'MAX' : `+${karyokuMax - karyokuNow}`;
  return _react["default"].createElement(KaryokuCell, _extends({}, props, {
    max: karyokuNow >= karyokuMax
  }), _react["default"].createElement("span", null, `${karyoku}/`, _react["default"].createElement("span", {
    style: {
      fontSize: '80%'
    }
  }, karyokuString)));
};

const RaisouCell = (0, _styledComponents["default"])(Cell).withConfig({
  displayName: "cells__RaisouCell",
  componentId: "hq8fj8-2"
})(["background-color:", ";"], props => props.max && (0, _polished.rgba)(props.theme.COBALT3, TRANSPARENCY));

const Raisou = ({
  ship,
  ...props
}) => {
  const {
    raisou,
    raisouMax,
    raisouNow
  } = ship;
  const raisouString = raisouNow >= raisouMax ? 'MAX' : `+${raisouMax - raisouNow}`;
  return _react["default"].createElement(RaisouCell, _extends({}, props, {
    max: raisouNow >= raisouMax
  }), _react["default"].createElement("span", null, `${raisou}/`, _react["default"].createElement("span", {
    style: {
      fontSize: '80%'
    }
  }, raisouString)));
};

const TaikuCell = (0, _styledComponents["default"])(Cell).withConfig({
  displayName: "cells__TaikuCell",
  componentId: "hq8fj8-3"
})(["background-color:", ";"], props => props.max && (0, _polished.rgba)(props.theme.ORANGE3, TRANSPARENCY));

const Taiku = ({
  ship,
  ...props
}) => {
  const {
    taiku,
    taikuMax,
    taikuNow
  } = ship;
  const taikuString = taikuNow >= taikuMax ? 'MAX' : `+${taikuMax - taikuNow}`;
  return _react["default"].createElement(TaikuCell, _extends({}, props, {
    max: taikuNow >= taikuMax
  }), _react["default"].createElement("span", null, `${taiku}/`, _react["default"].createElement("span", {
    style: {
      fontSize: '80%'
    }
  }, taikuString)));
};

const SoukouCell = (0, _styledComponents["default"])(Cell).withConfig({
  displayName: "cells__SoukouCell",
  componentId: "hq8fj8-4"
})(["background-color:", ";"], props => props.max && (0, _polished.rgba)(props.theme.INDIGO3, TRANSPARENCY));

const Soukou = ({
  ship,
  ...props
}) => {
  const {
    soukou,
    soukouMax,
    soukouNow
  } = ship;
  const soukouString = soukouNow >= soukouMax ? 'MAX' : `+${soukouMax - soukouNow}`;
  return _react["default"].createElement(SoukouCell, _extends({}, props, {
    max: soukouNow >= soukouMax
  }), _react["default"].createElement("span", null, `${soukou}/`, _react["default"].createElement("span", {
    style: {
      fontSize: '80%'
    }
  }, soukouString)));
};

const LuckyCell = (0, _styledComponents["default"])(Cell).withConfig({
  displayName: "cells__LuckyCell",
  componentId: "hq8fj8-5"
})(["background-color:", ";"], props => props.max && (0, _polished.rgba)(props.theme.LIME3, TRANSPARENCY));

const Lucky = ({
  ship,
  ...props
}) => {
  const {
    lucky,
    luckyMax,
    luckyNow
  } = ship;
  const luckyString = luckyNow >= luckyMax ? 'MAX' : `+${luckyMax - luckyNow}`;
  return _react["default"].createElement(LuckyCell, _extends({}, props, {
    max: luckyNow >= luckyMax
  }), _react["default"].createElement("span", null, `${lucky}/`, _react["default"].createElement("span", {
    style: {
      fontSize: '80%'
    }
  }, luckyString)));
};

const Kaihi = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.kaihi || 'NA');

const Taisen = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, _react["default"].createElement("span", null, ship.taisen, ship.kyouka[6] > 0 && _react["default"].createElement("sup", null, "+", ship.kyouka[6])));

const Sakuteki = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.sakuteki);

const RepairTime = ({
  ship,
  ...props
}) => {
  const {
    nowhp,
    maxhp,
    repairtime,
    lv,
    typeId,
    inDock
  } = ship;
  return _react["default"].createElement(Cell, props, repairtime && _react["default"].createElement(_overlay.Tooltip, {
    content: `1HP : ${(0, _tools.resolveTime)((0, _utils.getTimePerHP)(lv, typeId) / 1000)}`
  }, _react["default"].createElement("span", null, inDock && _react["default"].createElement(_reactFontawesome["default"], {
    name: "bath"
  }), (0, _tools.resolveTime)(repairtime))));
};

const Equipment = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, _react["default"].createElement(_slotitems.Slotitems, {
  slot: ship.slot,
  exslot: ship.exslot
}));

const Lock = ({
  ship,
  ...props
}) => _react["default"].createElement(Cell, props, ship.locked === 1 ? _react["default"].createElement(_reactFontawesome["default"], {
  name: "lock"
}) : ' ');

const Cells = {
  cond: Cond,
  equipment: Equipment,
  hp: Hp,
  id: Id,
  kaihi: Kaihi,
  karyoku: Karyoku,
  lock: Lock,
  lucky: Lucky,
  lv: Lv,
  name: (0, _reactRedux.connect)(state => ({
    enableAvatar: enableAvatarConfigSelector(state)
  }))(Name),
  raisou: Raisou,
  repairtime: RepairTime,
  sakuteki: Sakuteki,
  soku: Soku,
  soukou: Soukou,
  taiku: Taiku,
  taisen: Taisen,
  type: Type
};
exports.Cells = Cells;