"use strict";

exports.__esModule = true;
exports.TableView = void 0;

var _lodash = require("lodash");

var _react = _interopRequireWildcard(require("react"));

var _reactRedux = require("react-redux");

var _reactVirtualizedAutoSizer = _interopRequireDefault(require("react-virtualized-auto-sizer"));

var _reactWindow = require("react-window");

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _windowEnv = require("views/components/etc/window-env");

var _selectors = require("../selectors");

var _cells = require("./cells");

var _columnsConfig = require("./columns-config");

var _titleCell = require("./title-cell");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const TYPES = (0, _lodash.map)(_columnsConfig.ColumnsConfig, 'name');
const TITLES = (0, _lodash.map)(_columnsConfig.ColumnsConfig, 'title');
const SORTABLES = (0, _lodash.map)(_columnsConfig.ColumnsConfig, 'sortable');
const CENTER_ALIGNS = (0, _lodash.map)(_columnsConfig.ColumnsConfig, 'center'); // width will always unshift 1 extra element for row index

const WIDTHS = [40].concat((0, _lodash.map)(_columnsConfig.ColumnsConfig, 'width'));
const ROW_HEIGHT = 35;

const GridWrapper = _styledComponents["default"].div.withConfig({
  displayName: "table__GridWrapper",
  componentId: "sc-11dbvpk-0"
})(["flex:1;margin:0;padding:0;"]);

const Spacer = _styledComponents["default"].div.withConfig({
  displayName: "table__Spacer",
  componentId: "sc-11dbvpk-1"
})(["height:35px;"]);

const GridHeader = (0, _styledComponents["default"])(_reactWindow.VariableSizeGrid).withConfig({
  displayName: "table__GridHeader",
  componentId: "sc-11dbvpk-2"
})(["::-webkit-scrollbar{height:0;width:0;}::-webkit-scrollbar-thumb{height:0;width:0;}"]);

class ShipInfoTableAreaBase extends _react.Component {
  constructor(_props) {
    super(_props);
    this.tableWidth = (0, _lodash.sum)(WIDTHS);
    this.grid = (0, _react.createRef)();
    this.gridHeader = (0, _react.createRef)();
    this.tableArea = (0, _react.createRef)();
    this.activeColumn = -1;
    this.activeRow = -1;
    this.columnStopIndex = 0;
    this.rowStopIndex = 0;
    this.tableAreaWidth = void 0;
    this.handleClickTitle = (0, _lodash.memoize)(title => () => {
      if (this.props.sortName !== title) {
        const order = title === 'id' || title === 'type' || title === 'name' ? 1 : 0;
        this.saveSortRules(title, order);
      } else {
        this.saveSortRules(this.props.sortName, (this.props.sortOrder + 1) % 2);
      }
    });
    this.handleResize = (0, _lodash.debounce)(({
      width
    }) => {
      this.tableAreaWidth = width;

      if (this.grid.current) {
        this.grid.current.resetAfterColumnIndex(0);
      }

      if (this.gridHeader.current) {
        this.gridHeader.current.resetAfterColumnIndex(0);
      }
    }, 100);

    this.getItemKey = ({
      columnIndex,
      data,
      rowIndex
    }) => rowIndex === 0 ? `title-${columnIndex}` : `${data[rowIndex].id}-${columnIndex}`;

    this.onContextMenu = () => this.setState({
      activeColumn: -1,
      activeRow: -1
    });

    this.onClickFactory = ({
      columnIndex,
      rowIndex
    }) => () => {
      const {
        activeColumn,
        activeRow
      } = this.state;
      const off = activeColumn === columnIndex && activeRow === rowIndex;
      this.setState({
        activeColumn: off ? -1 : columnIndex,
        activeRow: off ? -1 : rowIndex
      });
    };

    this.getColumnWidth = index => {
      // 20: magic number, seems it need to be greater than 16
      const width = (0, _lodash.floor)((WIDTHS[index] || 40) * (this.tableAreaWidth - 20 > this.tableWidth ? (this.tableAreaWidth - 20) / this.tableWidth : 1));
      return width;
    };

    this.getRowHeight = () => ROW_HEIGHT;

    this.saveSortRules = (name, order) => {
      window.config.set('plugin.ShipInfo.sortName', name);
      window.config.set('plugin.ShipInfo.sortOrder', order);
    };

    this.cellRenderer = ({
      columnIndex,
      rowIndex,
      style
    }) => {
      const isHighlighteds = (columnIndex === this.state.activeColumn || rowIndex === this.state.activeRow) && !(columnIndex === 0 && rowIndex !== this.state.activeRow) && !(rowIndex === 0 && columnIndex !== this.state.activeColumn);
      const props = {
        centerAlign: CENTER_ALIGNS[columnIndex - 1],
        isEven: rowIndex % 2 === 1,
        isHighlighteds,
        onClick: this.onClickFactory({
          columnIndex,
          rowIndex
        }),
        onContextMenu: this.onContextMenu,
        style
      };
      let content;

      if (columnIndex === 0) {
        content = _react["default"].createElement(_cells.Cell, props, rowIndex + 1);
      } else {
        const index = columnIndex - 1;
        const {
          ids,
          ships
        } = this.props;
        const ship = ships[ids[rowIndex]];
        const Cell = _cells.Cells[TYPES[index]];
        content = _react["default"].createElement(Cell, _extends({
          ship: ship
        }, props));
      }

      return content;
    };

    this.titleRenderer = ({
      columnIndex,
      style,
      ...props
    }) => {
      if (columnIndex === 0) {
        return _react["default"].createElement("div", _extends({
          style: style
        }, props));
      }

      const {
        sortName,
        sortOrder
      } = this.props;
      const index = columnIndex - 1;
      return _react["default"].createElement(_titleCell.TitleCell, _extends({}, props, {
        style: style,
        title: TITLES[index],
        sortable: SORTABLES[index],
        centerAlign: CENTER_ALIGNS[index],
        sorting: sortName === TYPES[index],
        up: Boolean(sortOrder),
        onClick: this.handleClickTitle(TYPES[index])
      }));
    };

    this.handleScroll = ({
      scrollLeft
    }) => {
      if (this.gridHeader.current) {
        this.gridHeader.current.scrollTo({
          scrollLeft,
          scrollTop: 0
        });
      }
    };

    this.onClickFactory = (0, _lodash.memoize)(this.onClickFactory);
    this.state = {
      activeColumn: -1,
      activeRow: -1
    };
    this.tableAreaWidth = _props.window.innerWidth;
  }

  render() {
    const {
      ids,
      ships,
      sortName,
      sortOrder,
      window
    } = this.props;
    const {
      activeRow,
      activeColumn
    } = this.state;
    const hasSpacer = process.platform === 'darwin' && !window.isMain;
    return _react["default"].createElement(GridWrapper, null, hasSpacer && _react["default"].createElement(Spacer, null), _react["default"].createElement(_reactVirtualizedAutoSizer["default"], {
      onResize: this.handleResize
    }, ({
      height,
      width
    }) => _react["default"].createElement(_react["default"].Fragment, null, _react["default"].createElement(GridHeader, {
      ref: this.gridHeader,
      columnCount: WIDTHS.length,
      columnWidth: this.getColumnWidth,
      height: 35,
      rowCount: 1,
      rowHeight: this.getRowHeight,
      width: width - 32,
      itemData: {
        sortName,
        sortOrder
      } // to trigger header's re-render

    }, this.titleRenderer), _react["default"].createElement(_reactWindow.VariableSizeGrid, {
      className: "ship-info-scrollable",
      ref: this.grid,
      columnCount: WIDTHS.length,
      columnWidth: this.getColumnWidth,
      height: height - (hasSpacer ? 75 : 40),
      itemData: (0, _lodash.map)(ids, id => ships[id]),
      itemKey: this.getItemKey,
      rowCount: ids.length,
      rowHeight: this.getRowHeight,
      width: width - 16,
      onScroll: this.handleScroll
    }, this.cellRenderer))));
  }

}

const sortNameSelector = state => (0, _lodash.get)(state.config, 'plugin.ShipInfo.sortName', 'lv');

const sortOrderSelector = state => (0, _lodash.get)(state.config, 'plugin.ShipInfo.sortOrder', 0);

const ShipInfoTableArea = (0, _reactRedux.connect)(state => ({
  ids: (0, _selectors.filterShipIdsSelector)(state),
  ships: (0, _selectors.allShipRowsMapSelector)(state),
  sortName: sortNameSelector(state),
  sortOrder: sortOrderSelector(state)
}))(ShipInfoTableAreaBase);

const TableView = props => _react["default"].createElement(_windowEnv.WindowEnv.Consumer, null, ({
  window
}) => _react["default"].createElement(ShipInfoTableArea, _extends({
  window: window
}, props)));

exports.TableView = TableView;